
/*
	AutoPlayer plugin for SoundPlay
	Copyright 2000 Mikol Ryon (ryon@ryon.co.nz)

	Permission is hereby granted to use this code for creating other
	SoundPlay-plugins.

	Based upon example from Marco Nelissen
	Register SoundPlay and pay Marco 'yer shareware fee. ;)
	

*/



#include <Roster.h>
#include <stdio.h>

#include <VolumeRoster.h>
#include <NodeMonitor.h>
#include <Volume.h>
#include <Directory.h>

#include "pluginproto.h"

static const char *PLUGINID="AutoPlayer";
static void *getplugin(void **data,const char *name, const char *, uint32, plugin_info * );
static void destroyplugin(void*,void *_loader);

// =======================================================================
// standard plugin stuff
static plugin_descriptor plugin={
	PLUGIN_INTERFACE_MAGIC,PLUGIN_INTERFACE_VERSION,
	PLUGINID,1, PLUGIN_IS_INTERFACE,

	"Auto player",
	"ver. 0.1 Pretty Basic at this point. 
No settings, just turn it on and 
it will play audio CD automagicaly 
as you mount them",
	NULL,
	NULL,
	&getplugin,
	&destroyplugin
};

static interface_plugin_ops plugin_ops={
	PLUGIN_FILTER_MAGIC, PLUGIN_FILTER_VERSION,
	NULL,
	NULL,
	NULL,
	NULL
};

static plugin_descriptor *plugs[]={
	&plugin,0
};

plugin_descriptor **get_plugin_list(void)
{
	return plugs;
}

class AutoPlayerWatcher: public BHandler
{

 
	private:
		SoundPlayController *ctrl;


	public:

		BVolumeRoster *fVolRoster;
		
		AutoPlayerWatcher(SoundPlayController *controller)
		{
			be_app->AddHandler(this);
			ctrl=controller;
			ctrl->Lock();
			ctrl->AddListener(this);
			ctrl->Unlock();

        	fVolRoster = new BVolumeRoster();
        	fVolRoster->Rewind();
        	BVolume vol;

        	BMessenger msgr(this);
        	be_roster->StartWatching(msgr, B_REQUEST_LAUNCHED | B_REQUEST_QUIT | B_REQUEST_ACTIVATED);
        	fVolRoster->StartWatching(msgr);

		}
		
		~AutoPlayerWatcher()
		{
			ctrl->Lock();
			ctrl->RemoveListener(this);
			ctrl->Unlock();
			be_app->RemoveHandler(this);
		}
		virtual void MessageReceived(BMessage *mes)
		{
			switch(mes->what)
			{
			

                //BVolumeRoster messages
                case B_NODE_MONITOR:
                {
                        int32 opcode;
                        if (mes->FindInt32("opcode", &opcode) != B_OK)
                                                        break;

                        //volume mounted message
                        if (opcode == B_DEVICE_MOUNTED) {
                                //find the device of the mounted volume
                                dev_t device = 0;
                                if (mes->FindInt32("new device", &device) != B_OK)
                                                        break;

                                BDirectory dir;
								BEntry entry;
								status_t err;
								entry_ref *dir_ref = new entry_ref;
								int32 pid=-1;
								PlaylistPtr playlist(pid);
								BVolume vol(device);
								
								
								if(vol.InitCheck() != B_OK) break;

								vol.GetRootDirectory(&dir);
								if (dir.InitCheck() != B_OK) break;
								
//								int x=dir.CountEntries();
//								printf("entries %d\n", x);


								if (dir.FindEntry("wav", &entry) != B_OK) {
									err = entry.InitCheck(); 
									printf("no wav dir, must not be musik\n");
								} else {
									entry.GetRef(dir_ref);
									ctrl->Lock();
									playlist=ctrl->AddPlaylist();
									ctrl->Unlock();
									
									playlist->Lock();
									playlist->MakeEmpty();
									playlist->AddDir(*dir_ref);
								//	playlist->Play();		//hack 'cuz it won't play the first file?
									playlist->PlayFile(playlist->IDForItemAt(0));

									playlist->Unlock();
				
								}													

                        }
                       
                }

			}
		}

};

// =======================================================================
// plugin struct functions implementation

static void *getplugin(void **data,const char *name, const char *, uint32, plugin_info *info )
{
	AutoPlayerWatcher *watcher=new AutoPlayerWatcher(info->controller);
	
	*data=watcher;
	return &plugin_ops;
}

static void destroyplugin(void *,void *data)
{
	AutoPlayerWatcher *watcher=(AutoPlayerWatcher*)data;
	delete watcher;
}


